/*
Plays multiple midi files from an SD Card
Only midi file on the SD Card
All files must be in root directory
File names must follow following format XXXXXXXX.mid
*/
#include <SdFat.h>
#include <sdios.h>
#include <MD_MIDIFile.h>

#define SERIAL_RATE 31250 // Midi standard serial rate is 31250 baud
#define ARRAY_SIZE(a) (sizeof(a)/sizeof((a)[0]))

SDFAT  SD;
File file;
File dir;
MD_MIDIFile SMF; // create an instance of a midi file
const byte SD_SELECT = 10; // Chip select for Sd card is pin 10
char midiName[13]; // character array to hold midi file names
const int buttonPin = 7;    // the number of the pushbutton pin
const int ledPin = 6;      // the number of the LED pin
bool playFile = false; // boolean variable indicating if a file should be played
int err; // error code returned from midi file load


// Called by the MIDIFile library when a file event needs to be processed
// thru the midi communications interface.
// This callback is set up in the setup() function.
void midiCallback(midi_event *pev){
  if ((pev->data[0] >= 0x80) && (pev->data[0] <= 0xe0)){  
    Serial.write(pev->data[0] | pev->channel);
    Serial.write(&pev->data[1], pev->size-1);
  }
  else {
    Serial.write(pev->data, pev->size);
  }
}

void setup() {
  pinMode(buttonPin, INPUT_PULLUP); // use built in pullup resistor with push button
  pinMode(ledPin, OUTPUT);
  digitalWrite(ledPin, LOW);
  
  Serial.begin(SERIAL_RATE);

  // Initialize SD Card
  if (!SD.begin(SD_SELECT, SPI_FULL_SPEED)){  
    while (true) ;
  }

  // Initialize MIDIFile
  SMF.begin(&SD);
  SMF.setMidiHandler(midiCallback);
  SMF.looping(false);
}

void loop() {
  // Wait for pushbutton input
  int reading = digitalRead(buttonPin);
  // If push button is depressed, load midi file and switch on LED
  if (reading == LOW) {
    playFile = true;
    digitalWrite(ledPin, HIGH); // Switch on LED
    }

  // play files son the SD card if playFile is true
  if(playFile == true){
    dir.open("/");
    while (file.openNext(&dir, O_RDONLY)) {
      file.getName(midiName,13);  // Use SdFat getfile name to get next midi file from SD Card
      file.close();
      
      char *nextTune = &midiName[0];  // create pointer to the filename retrieved by SdFat
      err = SMF.load(nextTune);

      // Play until end of file is reached  
      while (!SMF.isEOF()){
        SMF.getNextEvent();
      }

      SMF.close();
      delay(500);
    }
  }
  // Once files are finished playing, set flag to false and switch off LED
  playFile = false;
  digitalWrite(ledPin, LOW);
}